# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoDataflow_Action.ipynb.

# %% ../../nbs/classes/50_DomoDataflow_Action.ipynb 2
from __future__ import annotations

from enum import Enum
from dataclasses import dataclass
from typing import List

import datetime as dt
import domolibrary.utils.DictDot as util_dd
import domolibrary.utils.convert as ct

from nbdev.showdoc import patch_to

# %% auto 0
__all__ = [
    "DomoDataflow_Action_Type",
    "DomoAction",
    "DomoDataflow_Action",
    "DomoDataflow_ActionResult",
]


# %% ../../nbs/classes/50_DomoDataflow_Action.ipynb 4
class DomoDataflow_Action_Type(Enum):
    LoadFromVault = "LoadFromVault"
    PublishToVault = "PublishToVault"
    GenerateTableAction = "GenerateTableAction"


@dataclass
class DomoAction:
    id: str
    type: str = None
    name: str = None


@dataclass
class DomoDataflow_Action(DomoAction):
    datasource_id: str = None
    sql: str = None

    depends_on: List[str] = None
    parent_actions: List[dict] = None

    @classmethod
    def _from_json(cls, obj: dict):
        dd = obj

        if isinstance(dd, dict):
            dd = util_dd.DictDot(obj)

        tbl_name = dd.dataSource.name if dd.dataSource else None
        ds_id = dd.dataSource.guid if dd.dataSource else None

        return cls(
            type=dd.type,
            id=dd.id,
            name=dd.name or dd.targetTableName or dd.tableName or tbl_name,
            depends_on=dd.dependsOn,
            datasource_id=dd.dataSourceId or ds_id,
            sql=dd.selectStatement or dd.query,
        )


# %% ../../nbs/classes/50_DomoDataflow_Action.ipynb 5
@patch_to(DomoDataflow_Action)
def get_parents(self: DomoDataflow_Action, domo_actions: List[DomoDataflow_Action]):
    if self.depends_on and len(self.depends_on) > 0:
        self.parent_actions = [
            parent_action
            for depends_id in self.depends_on
            for parent_action in domo_actions
            if parent_action.id == depends_id
        ]

    if self.parent_actions:
        [
            parent.get_parents(domo_actions)
            for parent in self.parent_actions
            if parent.depends_on
        ]

    return self.parent_actions


# %% ../../nbs/classes/50_DomoDataflow_Action.ipynb 8
@dataclass
class DomoDataflow_ActionResult(DomoAction):
    is_success: bool = None
    rows_processed: int = None
    begin_time: dt.datetime = None
    end_time: dt.datetime = None
    duration_in_sec: int = None

    def __post_init__(self):
        if self.begin_time and self.end_time:
            self.duration_in_sec = (self.end_time - self.begin_time).total_seconds()

    @classmethod
    def _from_json(cls, obj: dict):
        return cls(
            id=obj.get("actionId"),
            type=obj.get("type"),
            is_success=obj.get("wasSuccessful"),
            begin_time=ct.convert_epoch_millisecond_to_datetime(
                obj.get("beginTime", None)
            ),
            end_time=ct.convert_epoch_millisecond_to_datetime(obj.get("endTime", None)),
            rows_processed=obj.get("rowsProcessed", None),
        )



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/client/95_Logger.ipynb.

# %% auto 0
__all__ = ["TracebackDetails", "get_traceback", "Logger"]

# %% ../../nbs/client/95_Logger.ipynb 2
import datetime as dt

from typing import Optional, List
from dataclasses import dataclass, field

import traceback

from nbdev.showdoc import patch_to


# %% ../../nbs/client/95_Logger.ipynb 5
@dataclass
class TracebackDetails:
    """result of _get_traceback_details function"""

    function_name: str
    file_name: str
    function_trail: str

    traceback_stack: [traceback.FrameSummary] = None
    parent_class: str = None

    def __init__(
        self,
        traceback_stack: [traceback.FrameSummary],
        parent_class=None,  # pass ParentClass.__name__
        debug_traceback: bool = False,
    ):
        self.function_trail = " -> ".join([line[2] for line in traceback_stack])

        self.function_name = traceback_stack[-1][2]
        self.file_name = traceback_stack[-1][0]
        self.parent_class = parent_class
        self.traceback_stack = traceback_stack


def get_traceback(
    root_module: str = "<module>",
    # drop entries from the top of stack to exclude the functions that retrieve the traceback
    num_stacks_to_drop=0,
    parent_class: str = None,
    debug_traceback: bool = False,
) -> TracebackDetails:  # returns a filtered list of FrameSummaries from traceback
    """method that retrieves traceback"""

    import traceback

    traceback_stack = traceback.extract_stack()

    # find the last module index
    module_index = 0

    for index, tb_line in enumerate(traceback_stack):
        function_name = tb_line[2]

        if function_name == root_module:
            module_index = index

    num_stacks_to_drop += 1  # adjust for init

    if module_index + num_stacks_to_drop >= len(traceback_stack) - 1:
        print("adjusting num_stacks_to_drop, consider revising `get_traceback` call")
        print(
            {
                "stack_length": len(traceback_stack),
                "module_index": module_index,
                "num_stacks_to_drop_passed": num_stacks_to_drop,
            }
        )
        num_stacks_to_drop -= 1

    filtered_traceback_stack = traceback_stack[module_index:-num_stacks_to_drop]

    if debug_traceback:
        print(
            {
                "len orig stack": len(traceback_stack),
                "len filtered stack": len(filtered_traceback_stack),
                "root_module_name": root_module,
                "root_module_index": module_index,
                "stacks_to_drop": num_stacks_to_drop,
            }
        )

    return TracebackDetails(
        traceback_stack=filtered_traceback_stack,
        parent_class=parent_class,
        debug_traceback=debug_traceback,
    )


# %% ../../nbs/client/95_Logger.ipynb 8
class Logger:
    """log class with user customizeable output method"""

    root_module: str
    app_name: str

    logs: List[dict]
    breadcrumb: Optional[list]

    entity_id: Optional[str]
    domo_instance: Optional[str]
    # function to call with write_logs method.
    output_fn: Optional[callable] = None

    def __init__(
        self,
        app_name: str,  # name of the app for grouping logs
        root_module: Optional[str] = "<module>",  # root module for stack trace
        output_fn: Optional[
            callable
        ] = None,  # function to call with write_logs method.
        entity_id: Optional[str] = None,
        domo_instance: Optional[str] = None,
    ):
        self.app_name = app_name
        self.output_fn = output_fn
        self.root_module = root_module
        self.logs = []
        self.breadcrumb = []
        self.domo_instance = domo_instance
        self.entity_id = entity_id

    def _add_crumb(self, crumb):
        if crumb not in self.breadcrumb:
            self.breadcrumb.append(crumb)

    def _remove_crumb(self, crumb):
        if crumb in self.breadcrumb:
            self.breadcrumb.remove(crumb)

    def get_traceback(
        self,
        root_module: str = "<module>",
        # drop entries from the top of stack to exclude the functions that retrieve the traceback
        num_stacks_to_drop=0,
        parent_class: str = None,
    ):
        parent_class = parent_class or self.__class__.__name__

        num_stacks_to_drop += 1

        return get_traceback(
            root_module=root_module,
            num_stacks_to_drop=num_stacks_to_drop,
            parent_class=parent_class,
        )


# %% ../../nbs/client/95_Logger.ipynb 13
@patch_to(Logger)
def _add_log(
    self: Logger,
    message: str,
    type_str: str,
    debug_log: bool = False,
    num_stacks_to_drop=3,
    entity_id: Optional[str] = None,
    domo_instance: Optional[str] = None,
) -> dict:
    """internal method to append message to log"""

    traceback_details = self.get_traceback(num_stacks_to_drop=num_stacks_to_drop)

    if debug_log:
        print(traceback_details.__dict__)

    new_row = {
        "date_time": dt.datetime.now(),
        "application": self.app_name,
        "log_type": type_str,
        "log_message": message,
        "breadcrumb": "->".join(self.breadcrumb),
        "domo_instance": domo_instance or self.domo_instance,
        "entity_id": entity_id or self.entity_id,
    }

    new_row.update(
        {
            "function_name": traceback_details.function_name,
            "file_name": traceback_details.file_name,
            "function_trail": traceback_details.function_trail,
        }
    )

    if debug_log:
        print(new_row)

    self.logs.append(new_row)

    return new_row


@patch_to(Logger)
def log_info(
    self: Logger,
    message,
    entity_id: Optional[str] = None,
    domo_instance: Optional[str] = None,
    debug_log=False,
    num_stacks_to_drop=3,
):
    """log an informational message"""
    return self._add_log(
        message=message,
        entity_id=entity_id,
        domo_instance=domo_instance,
        type_str="Info",
        num_stacks_to_drop=num_stacks_to_drop,
        debug_log=debug_log,
    )


@patch_to(Logger)
def log_error(
    self: Logger,
    message,
    entity_id: Optional[str] = None,
    domo_instance: Optional[str] = None,
    debug_log=False,
    num_stacks_to_drop=3,
):
    """log an error message"""

    return self._add_log(
        message=message,
        entity_id=entity_id,
        domo_instance=domo_instance,
        type_str="Error",
        num_stacks_to_drop=num_stacks_to_drop,
        debug_log=debug_log,
    )


@patch_to(Logger)
def log_warning(
    self: Logger,
    message,
    entity_id: Optional[str] = None,
    domo_instance: Optional[str] = None,
    debug_log=False,
    num_stacks_to_drop=3,
):
    """log a warning message"""

    return self._add_log(
        message=message,
        entity_id=entity_id,
        domo_instance=domo_instance,
        type_str="Warning",
        num_stacks_to_drop=num_stacks_to_drop,
        debug_log=debug_log,
    )


# %% ../../nbs/client/95_Logger.ipynb 16
@patch_to(Logger)
def output_log(self: Logger):
    """calls the user defined output function"""
    return self.output_fn(self.logs)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/convert.ipynb.

# %% auto 0
__all__ = [
    "convert_epoch_millisecond_to_datetime",
    "convert_datetime_to_epoch_millisecond",
    "convert_snake_to_pascal",
    "convert_str_to_snake_case",
    "InvalidEmail",
    "test_valid_email",
    "convert_string_to_bool",
    "ConcatDataframe_InvalidElement",
    "concat_list_dataframe",
]

# %% ../../nbs/utils/convert.ipynb 2
import datetime as dt
import pandas as pd
import re


# %% ../../nbs/utils/convert.ipynb 5
def convert_epoch_millisecond_to_datetime(epoch: int):
    """convert Epoch time with miliseconds to Date time"""
    return dt.datetime.fromtimestamp(epoch / 1000.0) if epoch else None


# %% ../../nbs/utils/convert.ipynb 6
def convert_datetime_to_epoch_millisecond(datetime: dt.datetime):
    """convert DateTime to Epoch time with Miliseconds"""
    return int(datetime.timestamp() * 1000) if datetime else None


# %% ../../nbs/utils/convert.ipynb 10
def convert_snake_to_pascal(clean_str):
    """converts 'snake_case_str' to 'snakeCaseStr'"""

    clean_str = clean_str.replace("_", " ").title().replace(" ", "")
    return clean_str[0].lower() + clean_str[1:]


# %% ../../nbs/utils/convert.ipynb 13
def convert_str_to_snake_case(text_str, is_only_alphanumeric: bool = False):
    """converts 'snake_case_str' to 'snakeCaseStr'"""

    text_str = text_str.replace(" ", "_").lower()

    if is_only_alphanumeric:
        text_str = re.sub(r"\W+", "", text_str)

    return text_str


# %% ../../nbs/utils/convert.ipynb 15
class InvalidEmail(Exception):
    def __init__(self, email):

        super().__init__(f'invalid email: "{email}" provided')


def test_valid_email(email):
    """tests if provided string is a for valid email"""

    pattern = r"\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,7}\b"

    # pass the regular expression
    # and the string into the fullmatch() method
    if re.fullmatch(pattern, email):
        return True

    else:
        raise InvalidEmail(email=email)


# %% ../../nbs/utils/convert.ipynb 18
def convert_string_to_bool(v):
    return v.lower() in ("yes", "true", "t", "1")


# %% ../../nbs/utils/convert.ipynb 20
class ConcatDataframe_InvalidElement(Exception):
    def __init__(self, elem):
        message = f"{type(elem)} passed into dataframe"
        super().__init__(message)


def concat_list_dataframe(df_ls: list[pd.DataFrame]) -> pd.DataFrame:
    """take a list of dataframes and collapse into one dataframe"""

    df = None
    for elem in df_ls:
        if not isinstance(elem, pd.DataFrame):
            raise ConcatDataframe_InvalidElement(elem)

        if len(elem.index) == 0:
            pass

        if df is None:
            df = elem

        else:
            df = pd.concat([df, elem], join="inner").reset_index(drop=True)
    return df



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoJupyter.ipynb.

# %% auto 0
__all__ = ["DomoJupyterWorkspace_Content", "DomoJupyterWorkspace"]

# %% ../../nbs/classes/50_DomoJupyter.ipynb 2
from ..routes.jupyter import JupyterAPI_Error, JupyterAPI_Error

# %% ../../nbs/classes/50_DomoJupyter.ipynb 3
import os
import json

from dataclasses import dataclass, field
from typing import Union, List
import datetime as dt

import httpx

import domolibrary.utils.DictDot as util_dd
from dateutil.parser import parse

import domolibrary.client.DomoAuth as dmda
import domolibrary.routes.jupyter as jupyter_routes


# import domolibrary.client.DomoError as de
# import domolibrary.utils.chunk_execution as ce

from nbdev.showdoc import patch_to


# %% ../../nbs/classes/50_DomoJupyter.ipynb 5
@dataclass
class DomoJupyterWorkspace_Content:
    name: str
    folder: str
    last_modified: dt.datetime
    file_type: str
    content: str

    auth: dmda.DomoJupyterAuth = field(repr=False)

    default_export_folder: str = "export"

    def __post_init__(self):
        dmda.test_is_jupyter_auth(self.auth)

        if self.folder.endswith(self.name):
            self.folder = self.folder.replace(self.name, "")

    @classmethod
    def _from_json(cls, obj: dict, auth: dmda.DomoJupyterAuth):
        dd = util_dd.DictDot(obj) if not isinstance(obj, util_dd.DictDot) else obj

        dc = cls(
            name=dd.name,
            folder=dd.path,
            last_modified=parse(dd.last_modified),
            file_type=dd.type,
            auth=auth,
            content=obj.get("content"),
        )

        return dc

    def export(
        self,
        output_folder: str = None,
        file_name: str = None,
    ):
        output_folder = output_folder or os.path.join(
            self.default_export_folder, self.folder
        )

        file_name = file_name or self.name

        if not os.path.exists(output_folder):
            print(output_folder)
            os.makedirs(output_folder)

        content_str = self.content
        if isinstance(self.content, dict):

            content_str = json.dumps(self.content)

        output_path = os.path.join(output_folder, file_name)
        with open(output_path, "w", encoding="utf-8") as f:
            f.write(content_str)
            f.close()

        return output_path

    async def update(
        self,
        jupyter_folder: str = None,
        jupyter_file_name: str = None,
        debug_api: bool = False,
    ):
        if jupyter_folder and jupyter_file_name:
            content_patuh = f"{jupyter_folder}/{jupyter_file_name}"

        if len(self.folder) > 0:
            content_path = f"{self.folder}/{self.name}"

        else:
            content_path = self.name

            if content_path.lower().startswith(self.default_export_folder.lower()):
                content_path = content_path.replace(self.default_export_folder, "")

        content_path = "/".join(os.path.normpath(content_path).split(os.sep))

        return await jupyter_routes.update_jupyter_file(
            auth=self.auth,
            content_path=content_path,
            new_content=self.content,
            debug_api=debug_api,
            debug_num_stacks_to_drop=2,
            parent_class=self.__class__.__name__,
        )


# %% ../../nbs/classes/50_DomoJupyter.ipynb 7
@dataclass
class DomoJupyterWorkspace:
    auth: dmda.DomoJupyterAuth = field(repr=False)
    id: str
    name: str
    description: str

    created_dt: dt.datetime
    updated_dt: dt.datetime
    last_run_dt: dt.datetime

    # owner
    # cpu
    # memory

    instances: List[dict] = None
    input_configuration: list[dict] = None
    output_configuration: list[dict] = None
    account_configuration: list[dict] = None
    collection_configuration: list[dict] = None
    fileshare_configuration: list[dict] = None

    content: List[DomoJupyterWorkspace_Content] = field(default=None)

    jupyter_token: str = None
    service_location: str = None
    service_prefix: str = None

    def __post_init__(self):
        self._update_auth_params()

    def _update_auth_params(self):
        if self.instances:
            res = jupyter_routes.parse_instance_service_location_and_prefix(
                self.instances[0], self.auth.domo_instance
            )
            self.service_location = res["service_location"]
            self.service_prefix = res["service_prefix"]

        if self.service_location and self.service_prefix and self.jupyter_token:
            self.update_auth()

    def update_auth(
        self, service_location=None, service_prefix=None, jupyter_token=None
    ):

        self.service_location = service_location or self.service_location
        self.service_prefix = service_prefix or self.service_prefix
        self.jupyer_token = jupyter_token or self.jupyter_token

        if isinstance(self.auth, dmda.DomoFullAuth):
            self.auth = dmda.DomoJupyterFullAuth.convert_auth(
                auth=self.auth,
                service_location=self.service_location,
                jupyter_token=self.jupyter_token,
                service_prefix=self.service_prefix,
            )

        if isinstance(self.auth, dmda.DomoTokenAuth):
            self.auth = dmda.DomoJupyterTokenAuth.convert_auth(
                auth=self.auth,
                service_location=self.service_location,
                jupyter_token=self.jupyter_token,
                service_prefix=self.service_prefix,
            )

        self.auth.service_location = self.service_location
        self.auth.service_prefix = self.service_prefix
        self.auth.jupyer_token = self.jupyter_token

    @classmethod
    def _from_json(
        cls,
        obj,
        auth,
        jupyter_token: str = None,
    ):
        domo_workspace = cls(
            auth=auth,
            id=obj["id"],
            name=obj["name"],
            description=obj["description"],
            created_dt=obj["created"],
            updated_dt=obj["updated"],
            last_run_dt=obj["lastRun"],
            instances=obj["instances"],
            output_configuration=obj["outputConfiguration"],
            account_configuration=obj["accountConfiguration"],
            fileshare_configuration=obj["collectionConfiguration"],
            jupyter_token=jupyter_token,
        )
        return domo_workspace


# %% ../../nbs/classes/50_DomoJupyter.ipynb 8
@patch_to(DomoJupyterWorkspace, cls_method=True)
async def get_by_id(
    cls,
    workspace_id,
    auth: dmda.DomoAuth,  # this API does not require the jupyter_token, but activities inside the workspace will require additional authentication
    jupyter_token=None,
    return_raw: bool = False,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):

    res = await jupyter_routes.get_jupyter_workspace_by_id(
        workspace_id=workspace_id,
        auth=auth,
        session=session,
        debug_api=debug_api,
        parent_class=cls.__name__,
    )

    if return_raw:
        return res

    return cls._from_json(auth=auth, obj=res.response, jupyter_token=jupyter_token)


# %% ../../nbs/classes/50_DomoJupyter.ipynb 10
@patch_to(DomoJupyterWorkspace)
async def get_content(
    self,
    debug_api: bool = False,
    return_raw: bool = False,
    is_recursive: bool = True,
    content_path: str = "",
):
    res = await jupyter_routes.get_content(
        auth=self.auth,
        debug_api=debug_api,
        content_path=content_path,
        debug_num_stacks_to_drop=2,
        parent_class=self.__class__.__name__,
        is_recursive=is_recursive,
        return_raw=return_raw,
    )

    if return_raw:
        return res

    return [
        DomoJupyterWorkspace_Content._from_json(obj, auth=self.auth)
        for obj in res.response
    ]
